use NRT::Test;
use NRT::Helper;
use NRT::User;

package Batch;

sub new {
    my ($classe, %config) = @_;
    my $this = {};
    $classe = ref($classe) || $classe;
    bless ( $this, $classe );
    $this->{config} = {%config} ;
    $this->{tester} = Test->new( %config );
    $this->{users} = [];
    $this->{users_index} = 0;
    return $this;
}

sub test_rq_empty {
  my $this = shift;
  my $rq = $this->{tester}->get_code();  
  return $this->{tester}->assertEqual( $rq, "404 Not Found" );
}

sub test_user {
  my $this = shift;
  my $user1_index = $this->add_user(
    $this->{config}{Authentification}{Email},
    $this->{config}{Authentification}{Username},
    $this->{config}{Authentification}{Password} );
  $this->test_user_authentification( $user1_index );
  $this->test_user_already_created( $user1_index );

  my $user2_index = $this->add_user(
    $this->{config}{User}{Email},
    $this->{config}{User}{Username},
    $this->{config}{User}{Password} );
  return $this->test_user_create( $user2_index );
}

sub test_user_authentification {
  my ($this, $index) = @_;
  my $auth = $this->{users}[ $index ]->authentification;
  return $this->{tester}->assertEqual( $auth->{email}, $this->{config}{Authentification}{Email} );
}

sub test_user_create {
  my ($this, $index) = @_;
  return $this->create_user( $index );
}

sub test_user_already_created {
  my ($this, $index) = @_;
  my $creation = $this->create_user( $index );
  return $this->{tester}->assertNotEqual( $creation, "422 Unprocessable Entity");
}

sub create_user {
  my ($this, $index) = @_;
  return $this->{users}[ $index ]->create;  
}

sub add_user {
  my ($this, $email, $username, $password) = @_ ;
  push(
    @{$this->{users}},
      User->new( $email, $username, $password, $this->{tester})) ;
  return $this->{users_index}++;
}

sub run {
  my $this = shift;
  $this->test_rq_empty();
  return $this->test_user();
  $this->brief();
}

sub brief {
  my $this = shift;;
  print "\nResults :\n";
  Helper::printColor( "bright_green", "Tests succeed : ".$this->{tester}->{countSuccess}."\n" );
  Helper::printColor( "bright_red", "Tests failed  : ".$this->{tester}->{countFailure}."\n" );
}

return 1;
